@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         Exif.pm
 #
-# Description:  Definitions for EXIF tags
+# Description:  Read EXIF meta information
 #
 # Revisions:    11/25/2003 - P. Harvey Created
 #               02/06/2004 - P. Harvey Moved processing functions from ExifTool
@@ -9,7 +9,8 @@
 #               11/11/2004 - P. Harvey Split off maker notes into MakerNotes.pm
 #               12/13/2004 - P. Harvey Added AUTOLOAD to load write routines
 #
-# References:   1) http://partners.adobe.com/asn/developer/pdfs/tn/TIFF6.pdf
+# References:   0) http://www.exif.org/Exif2-2.PDF
+#               1) http://partners.adobe.com/asn/developer/pdfs/tn/TIFF6.pdf
 #               2) http://www.adobe.com/products/dng/pdfs/dng_spec.pdf
 #               3) http://www.awaresystems.be/imaging/tiff/tifftags.html
 #               4) http://www.remotesensing.org/libtiff/TIFFTechNote2.html
@@ -29,7 +30,7 @@ use vars qw($VERSION $AUTOLOAD @formatSize @formatName %formatNumber
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::MakerNotes;
 
-$VERSION = '1.69';
+$VERSION = '1.74';
 
 sub ProcessExif($$$);
 sub WriteExif($$$);
@@ -51,7 +52,7 @@ sub RebuildMakerNotes($$$);
 %formatNumber = (
     'int8u'         => 1,   # BYTE
     'string'        => 2,   # ASCII
-    'binary'        => 2,   # (binary is the same as string in Perl)
+    'binary'        => 2,   # (binary is the same as string in Perl, but watch out for UTF8)
     'int16u'        => 3,   # SHORT
     'int32u'        => 4,   # LONG
     'rational32u'   => 5,   # RATIONAL
@@ -163,7 +164,7 @@ sub RebuildMakerNotes($$$);
     0xfe => {
         Name => 'SubfileType',
         # set priority directory if this is the full resolution image
-        ValueConv => '$val == 0 and $self->SetPriorityDir(); $val',
+        RawConv => '$self->SetPriorityDir() if $val == 0; $val',
         PrintConv => {
             0 => 'Full-resolution Image',
             1 => 'Reduced-resolution image',
@@ -178,7 +179,7 @@ sub RebuildMakerNotes($$$);
     0xff => {
         Name => 'OldSubfileType',
         # set priority directory if this is the full resolution image
-        ValueConv => '$val == 1 and $self->SetPriorityDir(); $val',
+        RawConv => '$self->SetPriorityDir() if $val == 1; $val',
         PrintConv => {
             1 => 'Full-resolution image',
             2 => 'Reduced-resolution image',
@@ -239,7 +240,7 @@ sub RebuildMakerNotes($$$);
         Groups => { 2 => 'Camera' },
         DataMember => 'CameraMake',
         # save this value as an ExifTool member variable
-        ValueConv => '$self->{CameraMake} = $val',
+        RawConv => '$self->{CameraMake} = $val',
     },
     0x110 => {
         Name => 'Model',
@@ -247,7 +248,7 @@ sub RebuildMakerNotes($$$);
         Groups => { 2 => 'Camera' },
         DataMember => 'CameraModel',
         # save this value as an ExifTool member variable
-        ValueConv => '$self->{CameraModel} = $val',
+        RawConv => '$self->{CameraModel} = $val',
     },
     0x111 => [
         {
@@ -489,6 +490,7 @@ sub RebuildMakerNotes($$$);
         Groups => { 1 => 'GlobParamIFD' },
         Flags => 'SubIFD',
         SubDirectory => {
+            DirName => 'GlobParamIFD',
             Start => '$val',
         },
     },
@@ -664,6 +666,7 @@ sub RebuildMakerNotes($$$);
         Name => 'ApplicationNotes',
         # this could be an XMP block
         SubDirectory => {
+            DirName => 'XMP',
             TagTable => 'Image::ExifTool::XMP::Main',
         },
     },
@@ -701,6 +704,7 @@ sub RebuildMakerNotes($$$);
     0x83bb => {
         Name => 'IPTC-NAA',
         SubDirectory => {
+            DirName => 'IPTC',
             TagTable => 'Image::ExifTool::IPTC::Main',
         },
     },
@@ -729,18 +733,27 @@ sub RebuildMakerNotes($$$);
     0x8568 => {
         Name => 'IPTC-NAA2',
         SubDirectory => {
-            TagTable => 'Image::ExifTool::IPTC::Main',
             DirName => 'IPTC2',     # change name because this isn't the IPTC we want to write
+            TagTable => 'Image::ExifTool::IPTC::Main',
         },
     },
     0x85d8 => {
         Name => 'ModelTransform',
         Groups => { 2 => 'Location' },
     },
+    0x8606 => {
+        Name => 'LeafData',
+        Format => 'undef',    # avoid converting huge block to string of int8u's!
+        SubDirectory => {
+            DirName => 'LeafIFD',
+            TagTable => 'Image::ExifTool::Leaf::Main',
+        },
+    },
     0x8649 => {
         Name => 'PhotoshopSettings',
         Format => 'binary',
         SubDirectory => {
+            DirName => 'Photoshop',
             TagTable => 'Image::ExifTool::Photoshop::Main',
         },
     },
@@ -749,6 +762,7 @@ sub RebuildMakerNotes($$$);
         Groups => { 1 => 'ExifIFD' },
         Flags => 'SubIFD',
         SubDirectory => {
+            DirName => 'ExifIFD',
             Start => '$val',
         },
     },
@@ -796,6 +810,7 @@ sub RebuildMakerNotes($$$);
         Groups => { 1 => 'GPS' },
         Flags => 'SubIFD',
         SubDirectory => {
+            DirName => 'GPS',
             TagTable => 'Image::ExifTool::GPS::Main',
             Start => '$val',
         },
@@ -813,6 +828,16 @@ sub RebuildMakerNotes($$$);
     0x885c => 'FaxRecvParams', #9
     0x885d => 'FaxSubAddress', #9
     0x885e => 'FaxRecvTime', #9
+    0x888a => { #PH
+        Name => 'LeafSubIFD',
+        Format => 'int32u',     # Leaf incorrectly uses 'undef' format!
+        Groups => { 1 => 'LeafSubIFD' },
+        Flags => 'SubIFD',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Leaf::SubIFD',
+            Start => '$val',
+        },
+    },
     0x9000 => 'ExifVersion',
     0x9003 => {
         Name => 'DateTimeOriginal',
@@ -1025,6 +1050,7 @@ sub RebuildMakerNotes($$$);
         Flags => 'SubIFD',
         Description => 'Interoperability Offset',
         SubDirectory => {
+            DirName => 'InteropIFD',
             Start => '$val',
         },
     },
@@ -1194,70 +1220,11 @@ sub RebuildMakerNotes($$$);
     0xa480 => 'GDALMetadata', #3
     0xa481 => 'GDALNoData', #3
     0xa500 => 'Gamma',
-    # 0xc350 thru 0xc41a plus 0xc46c,0xc46e are Kodak APP3 tags (ref 8)
-    0xc350 => {
-        Name => 'FilmProductCode',
-        Notes => 'tags 0xc350-0xc41a, 0xc46c, 0xc46e are Kodak APP3 "Meta" tags',
-    },
-    0xc351 => 'ImageSourceEK',
-    0xc352 => 'CaptureConditionsPAR',
-    0xc353 => {
-        Name => 'CameraOwner',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
-    },
-    0xc354 => {
-        Name => 'SerialNumber',
-        Groups => { 2 => 'Camera' },
-    },
-    0xc355 => 'UserSelectGroupTitle',
-    0xc356 => 'DealerIDNumber',
-    0xc357 => 'CaptureDeviceFID',
-    0xc358 => 'EnvelopeNumber',
-    0xc359 => 'FrameNumber',
-    0xc35a => 'FilmCategory',
-    0xc35b => 'FilmGencode',
-    0xc35c => 'ModelAndVersion',
-    0xc35d => 'FilmSize',
-    0xc35e => 'SBA_RGBShifts',
-    0xc35f => 'SBAInputImageColorspace',
-    0xc360 => 'SBAInputImageBitDepth',
-    0xc361 => 'SBAExposureRecord',
-    0xc362 => 'UserAdjSBA_RGBShifts',
-    0xc363 => 'ImageRotationStatus',
-    0xc364 => 'RollGuidElements',
-    0xc365 => 'MetadataNumber',
-    0xc366 => 'EditTagArray',
-    0xc367 => 'Magnification',
-    0xc36c => 'NativeXResolution',
-    0xc36d => 'NativeYResolution',
-    0xc36e => {
-        Name => 'KodakEffectsIFD',
-        Flags => 'SubIFD',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Kodak::SpecialEffects',
-            Start => '$val',
-        },
-    },
-    0xc36f => {
-        Name => 'KodakBordersIFD',
-        Flags => 'SubIFD',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Kodak::Borders',
-            Start => '$val',
-        },
-    },
-    0xc37a => 'NativeResolutionUnit',
-    0xc418 => 'SourceImageDirectory',
-    0xc419 => 'SourceImageFileName',
-    0xc41a => 'SourceImageVolumeName',
-    # (end Kodak APP3 tags)
     0xc427 => 'OceScanjobDesc', #3
     0xc428 => 'OceApplicationSelector', #3
     0xc429 => 'OceIDNumber', #3
     0xc42a => 'OceImageLogic', #3
     0xc44f => 'Annotations', #7
-    0xc46c => 'PrintQuality', #8 (Kodak APP3)
-    0xc46e => 'ImagePrintStatus', #8 (Kodak APP3)
     0xc4a5 => {
         Name => 'PrintIM',
         # must set Writable here so this tag will be saved with MakerNotes option
@@ -1350,29 +1317,6 @@ sub RebuildMakerNotes($$$);
     # by ProcessExif().
 );
 
-# Kodak Sub-IFD's (ref 8) - (put them here for convenience)
-%Image::ExifTool::Kodak::SpecialEffects = (
-    GROUPS => { 0 => 'EXIF', 1 => 'KodakEffectsIFD', 2 => 'Image'},
-    0 => 'DigitalEffectsVersion',
-    1 => {
-        Name => 'DigitalEffectsName',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
-    },
-    2 => 'DigitalEffectsType',
-);
-%Image::ExifTool::Kodak::Borders = (
-    GROUPS => { 0 => 'EXIF', 1 => 'KodakBordersIFD', 2 => 'Image'},
-    0 => 'BordersVersion',
-    1 => {
-        Name => 'BorderName',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
-    },
-    2 => 'BorderID',
-    3 => 'BorderLocation',
-    4 => 'BorderType',
-    8 => 'WatermarkType',
-);
-
 # the Composite tags are evaluated last, and are used
 # to calculate values based on the other tags
 # (the main script looks for the special 'Composite' hash)
@@ -1400,8 +1344,7 @@ sub RebuildMakerNotes($$$);
             0 => 'FNumber',
             1 => 'ApertureValue',
         },
-        # convert to a float (could start with 'F' if it was a string value)
-        ValueConv => 'my $a=ToFloat($val[0]); $a || $val[1]',
+        ValueConv => '$val[0] || $val[1]',
         PrintConv => 'sprintf("%.1f", $val)',
     },
     FocalLength35efl => {
@@ -1478,11 +1421,33 @@ sub RebuildMakerNotes($$$);
             return join(' ',@v);
         },
         PrintConv => q{
+            $val =~ tr/,/./;    # in case locale is whacky
             my @v = split ' ', $val;
             $v[1] or return sprintf("inf (%.2f m - inf)", $v[0]);
             return sprintf("%.2f m (%.2f - %.2f)",$v[1]-$v[0],$v[0],$v[1]);
         },
     },
+    DateTimeCreated => { # used by IPTC, XMP, WAV, etc
+        Description => 'Date/Time Created',
+        Groups => { 2 => 'Time' },
+        Require => {
+            0 => 'DateCreated',
+            1 => 'TimeCreated',
+        },
+        ValueConv => '"$val[0] $val[1]"',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    # set the original date/time from DateTimeCreated if not set already
+    DateTimeOriginal => {
+        Condition => 'not defined($oldVal)',
+        Description => 'Shooting Date/Time',
+        Groups => { 2 => 'Time' },
+        Require => {
+            0 => 'DateTimeCreated',
+        },
+        ValueConv => '$val[0]',
+        PrintConv => '$valPrint[0]',
+    },
     ThumbnailImage => {
         Writable => 1,
         WriteCheck => '$self->CheckImage(\$val)',
@@ -1491,8 +1456,7 @@ sub RebuildMakerNotes($$$);
             1 => 'ThumbnailLength',
         },
         # retrieve the thumbnail from our EXIF data
-        ValueConv => 'Image::ExifTool::Exif::ExtractImage($self,$val[0],$val[1],"ThumbnailImage")',
-        ValueConvInv => '$val',
+        RawConv => 'Image::ExifTool::Exif::ExtractImage($self,$val[0],$val[1],"ThumbnailImage")',
     },
     PreviewImage => {
         Writable => 1,
@@ -1507,7 +1471,7 @@ sub RebuildMakerNotes($$$);
         WriteAlso => {
             PreviewImageValid => 'defined $val and length $val ? 1 : 0',
         },
-        ValueConv => q{
+        RawConv => q{
             return undef if defined $val[2] and not $val[2];
             return Image::ExifTool::Exif::ExtractImage($self,$val[0],$val[1],'PreviewImage');
         },
@@ -1520,7 +1484,7 @@ sub RebuildMakerNotes($$$);
             0 => 'JpgFromRawStart',
             1 => 'JpgFromRawLength',
         },
-        ValueConv => 'Image::ExifTool::Exif::ExtractImage($self,$val[0],$val[1],"JpgFromRaw")',
+        RawConv => 'Image::ExifTool::Exif::ExtractImage($self,$val[0],$val[1],"JpgFromRaw")',
         ValueConvInv => '$val',
     },
     PreviewImageSize => {
@@ -1530,26 +1494,6 @@ sub RebuildMakerNotes($$$);
         },
         ValueConv => '"$val[0]x$val[1]"',
     },
-    ProcessGeoTiff => {
-        Require => {
-            0 => 'GeoTiffDirectory',
-        },
-        Desire => {
-            1 => 'GeoTiffDoubleParams',
-            2 => 'GeoTiffAsciiParams',
-        },
-        ValueConv => q{
-            my $tagTable = GetTagTable("Image::ExifTool::GeoTiff::Main");
-            Image::ExifTool::GeoTiff::ProcessGeoTiff($self, $tagTable, $val[0], $val[1], $val[2]);
-            unless ($self->Options('Verbose')) {
-                # this is duplicate information so delete it unless verbose
-                $self->DeleteTag('GeoTiffDirectory');
-                $self->DeleteTag('GeoTiffDoubleParams');
-                $self->DeleteTag('GeoTiffAsciiParams');
-            }
-            return undef;       # don't generate a tag value
-        },
-    },
     SubSecDateTimeOriginal => {
         Description => 'Shooting Date/Time',
         Require => {
@@ -1576,6 +1520,12 @@ sub RebuildMakerNotes($$$);
     },
 );
 
+# table for unknown IFD entries
+%Image::ExifTool::Exif::Unknown = (
+    GROUPS => { 0 => 'EXIF', 1 => 'UnknownIFD', 2 => 'Image'},
+    WRITE_PROC => \&WriteExif,
+);
+
 # add our composite tags
 Image::ExifTool::AddCompositeTags('Image::ExifTool::Exif::Composite');
 
@@ -1661,6 +1611,10 @@ sub ConvertExifText($$)
     return $val if length($val) < 8;
     my $id = substr($val, 0, 8);
     my $str = substr($val, 8);
+    # by the EXIF spec, the string should be "UNICODE\0", but apparently Kodak
+    # sometimes uses "Unicode\0" in the APP3 "Meta" information.  But unfortunately
+    # Ricoh uses "Unicode\0" in the RR30 EXIF UserComment when the text is actually
+    # ASCII, so only recognize uppercase "UNICODE\0" here.
     if ($id eq "UNICODE\0") {
         # convert from unicode
         $str = $exifTool->Unicode2Byte($str);
@@ -1702,7 +1656,7 @@ sub ConvertParameter($)
     return 1 if $val =~ /\b(s|l|-)/i;
     # "hard", "high" or any positive number is a vail of 2
     return 2 if $val =~ /\b(h|\+|\d)/i;
-    return undef; 
+    return undef;
 }
 
 #------------------------------------------------------------------------------
@@ -1828,26 +1782,26 @@ sub ExtractImage($$$$)
 #------------------------------------------------------------------------------
 # Process EXIF directory
 # Inputs: 0) ExifTool object reference
-#         1) Pointer to tag table for this directory
-#         2) Reference to directory information hash
+#         1) Reference to directory information hash
+#         2) Pointer to tag table for this directory
 # Returns: 1 on success, otherwise returns 0 and sets a Warning
 sub ProcessExif($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
-    my $dataPt = $dirInfo->{DataPt};
-    my $dataPos = $dirInfo->{DataPos} || 0;
-    my $dataLen = $dirInfo->{DataLen};
-    my $dirStart = $dirInfo->{DirStart} || 0;
-    my $dirLen = $dirInfo->{DirLen} || $dataLen - $dirStart;
-    my $base = $dirInfo->{Base} || 0;
-    my $raf = $dirInfo->{RAF};
-    my $fixBase = $dirInfo->{FixBase};
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dataPos = $$dirInfo{DataPos} || 0;
+    my $dataLen = $$dirInfo{DataLen};
+    my $dirStart = $$dirInfo{DirStart} || 0;
+    my $dirLen = $$dirInfo{DirLen} || $dataLen - $dirStart;
+    my $base = $$dirInfo{Base} || 0;
+    my $raf = $$dirInfo{RAF};
+    my $fixBase = $$dirInfo{FixBase};
     my $success = 1;
     my $verbose = $exifTool->Options('Verbose');
     my $tagKey;
 
-    if ($dirInfo->{DirName} eq 'EXIF') {
-        $dirInfo->{DirName} = 'IFD0';
+    if ($$dirInfo{DirName} eq 'EXIF') {
+        $$dirInfo{DirName} = 'IFD0';
     }
     my ($numEntries, $dirEnd, $readFromFile);
     if ($dirStart < 0 or $dirStart > $dataLen-2) {
@@ -1875,17 +1829,17 @@ sub ProcessExif($$$)
             if ($raf->Seek($offset + $base, 0) and $raf->Read($buff,2) == 2) {
                 my $len = 12 * Get16u(\$buff,0);
                 # also read next IFD pointer if reading multiple IFD's
-                $len += 4 if $dirInfo->{Multi};
+                $len += 4 if $$dirInfo{Multi};
                 if ($raf->Read($buf2, $len) == $len) {
                     $buff .= $buf2;
                     # make copy of dirInfo since we're going to modify it
                     my %newDirInfo = %$dirInfo;
                     $dirInfo = \%newDirInfo;
                     # update directory parameters for the newly loaded IFD
-                    $dataPt = $dirInfo->{DataPt} = \$buff;
-                    $dataPos = $dirInfo->{DataPos} = $offset;
-                    $dataLen = $dirInfo->{DataLen} = $len + 2;
-                    $dirStart = $dirInfo->{DirStart} = 0;
+                    $dataPt = $$dirInfo{DataPt} = \$buff;
+                    $dataPos = $$dirInfo{DataPos} = $offset;
+                    $dataLen = $$dirInfo{DataLen} = $len + 2;
+                    $dirStart = $$dirInfo{DirStart} = 0;
                     $success = 1;
                 }
             }
@@ -1897,7 +1851,7 @@ sub ProcessExif($$$)
         $numEntries = Get16u($dataPt, $dirStart);
         $dirEnd = $dirStart + 2 + 12 * $numEntries;
     }
-    $verbose and $exifTool->VerboseDir($dirInfo->{DirName}, $numEntries);
+    $verbose and $exifTool->VerboseDir($$dirInfo{DirName}, $numEntries);
     my $bytesFromEnd = $dataLen - $dirEnd;
     if ($bytesFromEnd < 4) {
         unless ($bytesFromEnd==2 or $bytesFromEnd==0) {
@@ -1932,11 +1886,11 @@ sub ProcessExif($$$)
                 my $diff = $dirEnd + 4 - $valuePtr;
                 if ($diff) {
                     $valuePtr += $diff;
-                    $dirInfo->{Base} = $base = $base + $diff;
-                    $dirInfo->{DataPos} = $dataPos = $dataPos - $diff;
-                    $dirInfo->{Relative} = 1 if $base > $base+$dataPos+$dirStart;
+                    $$dirInfo{Base} = $base = $base + $diff;
+                    $$dirInfo{DataPos} = $dataPos = $dataPos - $diff;
+                    $$dirInfo{Relative} = 1 if $dataPos + $dirStart < 0;
                     if ($verbose) {
-                        my $rel = $dirInfo->{Relative} ? 'relative' : 'absolute';
+                        my $rel = $$dirInfo{Relative} ? 'relative' : 'absolute';
                         $exifTool->Warn("Adjusted $$dirInfo{DirName} base by $diff ($rel)");
                     }
                 }
@@ -2004,7 +1958,7 @@ sub ProcessExif($$$)
             if ($$tagInfo{Format}) {
                 $formatStr = $$tagInfo{Format};
                 # must adjust number of items for new format size
-                my $newNum = $formatNumber{$$tagInfo{Format}};
+                my $newNum = $formatNumber{$formatStr};
                 if ($newNum) {
                     $origFormStr = $formatName[$format] . '[' . $count . ']';
                     $format = $newNum;
@@ -2021,15 +1975,15 @@ sub ProcessExif($$$)
             my $fstr = $formatName[$format];
             $origFormStr and $fstr = "$origFormStr read as $fstr";
             $exifTool->VerboseInfo($tagID, $tagInfo,
-                Table  => $tagTablePtr,
-                Index  => $index,
-                Value  => $val,
-                DataPt => $valueDataPt,
-                Size   => $size,
-                Start  => $valuePtr,
-                Addr   => $valuePtr + $valueDataPos,
-                Format => $fstr,
-                Count  => $count,
+                Table   => $tagTablePtr,
+                Index   => $index,
+                Value   => $val,
+                DataPt  => $valueDataPt,
+                DataPos => $valueDataPos,
+                Size    => $size,
+                Start   => $valuePtr,
+                Format  => $fstr,
+                Count   => $count,
             );
             next unless $tagInfo;
         }
@@ -2101,7 +2055,7 @@ sub ProcessExif($$$)
                 if ($$subdir{OffsetPt}) {
                     SetByteOrder($newByteOrder);
                     #### eval OffsetPt ($valuePtr)
-                    $subdirStart += Get32u($dataPt, eval $$subdir{OffsetPt});
+                    $subdirStart += Get32u($subdirDataPt, eval $$subdir{OffsetPt});
                     SetByteOrder($oldByteOrder);
                 }
                 if ($subdirStart < 0 or $subdirStart + 2 > $subdirDataLen) {
@@ -2137,7 +2091,7 @@ sub ProcessExif($$$)
                                 $msg .= " (directory start $subdirStart is before EXIF start)";
                             } else {
                                 my $end = $subdirStart + $size;
-                                $msg .= " (directory end is $subdirStart but EXIF size is only $subdirDataLen)";
+                                $msg .= " (directory end is $end but EXIF size is only $subdirDataLen)";
                             }
                         }
                         $exifTool->Warn($msg);
@@ -2167,8 +2121,9 @@ sub ProcessExif($$$)
                     DirStart => $subdirStart,
                     DirLen   => $size,
                     RAF      => $raf,
-                    Parent   => $dirInfo->{DirName},
+                    Parent   => $$dirInfo{DirName},
                     FixBase  => $$subdir{FixBase},
+                    TagInfo  => $tagInfo,
                 );
                 # set directory IFD name from group name of family 1 in tag information if it exists
                 if ($$tagInfo{Groups}) {
@@ -2185,7 +2140,7 @@ sub ProcessExif($$$)
                     $exifTool->Warn("Invalid $tagStr data");
                 } else {
                     # process the subdirectory
-                    $ok = $exifTool->ProcessTagTable($newTagTable, \%subdirInfo, $$subdir{ProcessProc});
+                    $ok = $exifTool->ProcessDirectory(\%subdirInfo, $newTagTable, $$subdir{ProcessProc});
                 }
                 # print debugging information if there were errors
                 if (not $ok and $verbose > 1 and $subdirStart != $valuePtr) {
@@ -2210,13 +2165,11 @@ sub ProcessExif($$$)
                     DirStart => $valuePtr,
                     DirLen   => $size,
                     RAF      => $raf,
-                    Parent   => $dirInfo->{DirName},
+                    Parent   => $$dirInfo{DirName},
                     DirName  => 'MakerNotes',
                     TagInfo  => $tagInfo,
                 );
-                if ($$tagInfo{SubDirectory} and $tagInfo->{SubDirectory}->{FixBase}) {
-                    $makerDirInfo{FixBase} = 1;
-                }
+                $makerDirInfo{FixBase} = 1 if $$subdir{FixBase};
                 my $val2 = RebuildMakerNotes($exifTool, $newTagTable, \%makerDirInfo);
                 if (defined $val2) {
                     $val = $val2;
@@ -2239,11 +2192,14 @@ sub ProcessExif($$$)
         }
         # save the value of this tag
         $tagKey = $exifTool->FoundTag($tagInfo, $val);
-        $exifTool->SetTagExtra($tagKey, $dirInfo->{DirName}) if defined $tagKey;
+        # set the group 1 name for tags in main table
+        if (defined $tagKey and $tagTablePtr eq \%Image::ExifTool::Exif::Main) {
+            $exifTool->SetGroup1($tagKey, $$dirInfo{DirName});
+        }
     }
 
     # scan for subsequent IFD's if specified
-    if ($dirInfo->{Multi} and $bytesFromEnd >= 4) {
+    if ($$dirInfo{Multi} and $bytesFromEnd >= 4) {
         my $offset = Get32u($dataPt, $dirEnd);
         if ($offset) {
             my $subdirStart = $offset - $dataPos;
@@ -2255,8 +2211,9 @@ sub ProcessExif($$$)
             if ($newDirInfo{DirName} =~ /^IFD(\d+)$/) {
                 $newDirInfo{DirName} = 'IFD' . ($1 + 1);
             }
+            $exifTool->{INDENT} =~ s/..$//;
             $newDirInfo{DirStart} = $subdirStart;
-            $exifTool->ProcessTagTable($tagTablePtr, \%newDirInfo) or $success = 0;
+            $exifTool->ProcessDirectory(\%newDirInfo, $tagTablePtr) or $success = 0;
         }
     }
     return $success;
@@ -2268,7 +2225,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::Exif - Definitions for EXIF meta information
+Image::ExifTool::Exif - Read EXIF meta information
 
 =head1 SYNOPSIS
 
@@ -2290,6 +2247,8 @@ it under the same terms as Perl itself.
 
 =over 4
 
+=item L<http://www.exif.org/Exif2-2.PDF>
+
 =item L<http://partners.adobe.com/asn/developer/pdfs/tn/TIFF6.pdf>
 
 =item L<http://partners.adobe.com/public/developer/en/tiff/TIFFPM6.pdf>
