@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         GPS.pm
 #
-# Description:  Definitions for EXIF GPS tags
+# Description:  EXIF GPS meta information tags
 #
 # Revisions:    12/09/2003  - P. Harvey Created
 #------------------------------------------------------------------------------
@@ -10,8 +10,16 @@ package Image::ExifTool::GPS;
 
 use strict;
 use vars qw($VERSION);
+use Image::ExifTool::Exif;
 
-$VERSION = '1.06';
+$VERSION = '1.07';
+
+my %coordConv = (
+    ValueConv    => 'Image::ExifTool::GPS::ToDegrees($val)',
+    ValueConvInv => 'Image::ExifTool::GPS::ToDMS($self, $val)',
+    PrintConv    => 'Image::ExifTool::GPS::ToDMS($self, $val, 1)',
+    PrintConvInv => 'Image::ExifTool::GPS::ToDegrees($val)',
+);
 
 %Image::ExifTool::GPS::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -43,8 +51,7 @@ GPSAltitude and GPSAltitudeRef.
         Name => 'GPSLatitude',
         Writable => 'rational32u',
         Count => 3,
-        PrintConv => 'Image::ExifTool::GPS::DMS($val)',
-        PrintConvInv => '$_=$val;tr/-+0-9.\t/ /c;$_',
+        %coordConv,
     },
     0x0003 => {
         Name => 'GPSLongitudeRef',
@@ -59,8 +66,7 @@ GPSAltitude and GPSAltitudeRef.
         Name => 'GPSLongitude',
         Writable => 'rational32u',
         Count => 3,
-        PrintConv => 'Image::ExifTool::GPS::DMS($val)',
-        PrintConvInv => '$_=$val;tr/-+0-9.\t/ /c;$_',
+        %coordConv,
     },
     0x0005 => {
         Name => 'GPSAltitudeRef',
@@ -78,9 +84,10 @@ GPSAltitude and GPSAltitudeRef.
     },
     0x0007 => {
         Name => 'GPSTimeStamp',
+        Groups => { 2 => 'Time' },
         Writable => 'rational32u',
         Count => 3,
-        Groups => { 2 => 'Time' },
+        Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifTime($val)',
         ValueConvInv => '$val=~tr/:/ /;$val',
     },
@@ -167,8 +174,7 @@ GPSAltitude and GPSAltitudeRef.
         Name => 'GPSDestLatitude',
         Writable => 'rational32u',
         Count => 3,
-        PrintConv => 'Image::ExifTool::GPS::DMS($val)',
-        PrintConvInv => '$_=$val;tr/-+0-9.\t/ /c;$_',
+        %coordConv,
     },
     0x0015 => {
         Name => 'GPSDestLongitudeRef',
@@ -183,8 +189,7 @@ GPSAltitude and GPSAltitudeRef.
         Name => 'GPSDestLongitude',
         Writable => 'rational32u',
         Count => 3,
-        PrintConv => 'Image::ExifTool::GPS::DMS($val)',
-        PrintConvInv => '$_=$val;tr/-+0-9.\t/ /c;$_',
+        %coordConv,
     },
     0x0017 => {
         Name => 'GPSDestBearingRef',
@@ -227,9 +232,10 @@ GPSAltitude and GPSAltitudeRef.
     },
     0x001D => {
         Name => 'GPSDateStamp',
+        Groups => { 2 => 'Time' },
         Writable => 'string',
         Count => 11,
-        Groups => { 2 => 'Time' },
+        Shift => 'Time',
         ValueConv => 'Image::ExifTool::Exif::ExifDate($val)',
         ValueConvInv => '$val',
     },
@@ -255,17 +261,93 @@ GPSAltitude and GPSAltitudeRef.
         ValueConv => '"$val[0] $val[1]"',
         PrintConv => '$self->ConvertDateTime($val)',
     },
+    GPSPosition => {
+        Require => {
+            0 => 'GPSLatitude',
+            1 => 'GPSLongitude',
+        },
+        Desire => {
+            2 => 'GPSLatitudeRef',
+            3 => 'GPSLongitudeRef',
+        },
+        ValueConv => q{
+            foreach (0..1) {
+                # set sign from lat/long reference direction if it exists
+                next unless $val[$_+2];
+                $val[$_] = abs($val[$_]);
+                $val[$_] = -$val[$_] if $val[$_+2] =~ /^(S|W)/i;
+            }
+            return "$val[0] $val[1]";
+        },
+        PrintConv => q{
+            my $lat  = Image::ExifTool::GPS::ToDMS($self, $val[0], 1, 'N');
+            my $long = Image::ExifTool::GPS::ToDMS($self, $val[1], 1, 'E');
+            return "$lat, $long";
+        },
+    },
 );
 
 # add our composite tags
 Image::ExifTool::AddCompositeTags(\%Image::ExifTool::GPS::Composite);
 
-# Convert to DMS format
-sub DMS($)
+#------------------------------------------------------------------------------
+# Convert degrees to DMS, or whatever the current settings are
+# Inputs: 0) ExifTool reference, 1) Value in degrees,
+#         2) format code (0=no format, 1=CoordFormat, 2=XMP format)
+#         3) 'N' or 'E' if sign is significant and N/S/E/W should be added
+# Returns: DMS string
+sub ToDMS($$;$$)
 {
-    my $val = shift;
-    $val =~ s/^(\S+) (\S+) (.*)/$1 deg $2' $3"/;
-    return $val;
+    my ($exifTool, $val, $doPrintConv, $ref) = @_;
+    my ($fmt, $num);
+
+    if ($ref) {
+        if ($val < 0) {
+            $val = -$val;
+            $ref = {N => 'S', E => 'W'}->{$ref};
+        }
+        $ref = " $ref" unless $doPrintConv and $doPrintConv eq '2';
+    } else {
+        $ref = '';
+    }
+    if ($doPrintConv) {
+        if ($doPrintConv eq '1') {
+            $fmt = ($exifTool->Options('CoordFormat') || q{%d deg %d' %.2f"}) . $ref;
+        } else {
+            $fmt = "%d,%.2f$ref";   # use XMP standard format
+        }
+        # count the number of format specifiers
+        $num = ($fmt =~ tr/%/%/);
+    } else {
+        $num = 3;
+    }
+    my ($d, $m, $s);
+    $d = $val;
+    if ($num > 1) {
+        $d = int($d);
+        $m = ($val - $d) * 60;
+        if ($num > 2) {
+            $m = int($m);
+            $s = ($val - $d - $m / 60) * 3600;
+        }
+    }
+    return $doPrintConv ? sprintf($fmt, $d, $m, $s) : "$d $m $s$ref";
+}
+
+#------------------------------------------------------------------------------
+# Convert to decimal degrees
+# Inputs: 0) a string containing 1-3 decimal numbers and any amount of other garbage
+#         1) true if value should be negative if coordinate ends in 'S' or 'W'
+# Returns: Coordinate in degrees
+sub ToDegrees($;$)
+{
+    my ($val, $doSign) = @_;
+    # extract decimal values out of any other garbage
+    my ($d, $m, $s) = ($val =~ /((?:[+-]?)(?=\d|\.\d)\d*(?:\.\d*)?)/g);
+    my $deg = ($d || 0) + (($m || 0) + ($s || 0)/60) / 60;
+    # make negative if S or W coordinate
+    $deg = -$deg if $doSign and $val =~ /[^A-Z](S|W)$/i;
+    return $deg;
 }
 
 
@@ -275,7 +357,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::GPS - Definitions for GPS meta information
+Image::ExifTool::GPS - EXIF GPS meta information tags
 
 =head1 SYNOPSIS
 
