@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         Nikon.pm
 #
-# Description:  Definitions for Nikon EXIF Maker Notes
+# Description:  Nikon EXIF maker notes tags
 #
 # Revisions:    12/09/2003 - P. Harvey Created
 #               05/17/2004 - P. Harvey Added information from Joseph Heled
@@ -21,17 +21,19 @@
 #               7) Tom Christiansen private communication (tchrist@perl.com)
 #               8) Robert Rottmerhusen private communication
 #               9) http://members.aol.com/khancock/pilot/nbuddy/
-#              10) Werner Kober private communication (D2H, D2X, D100, D70)
+#              10) Werner Kober private communication (D2H, D2X, D100, D70, D200)
 #              11) http://www.rottmerhusen.com/objektives/lensid/nikkor.html
+#              12) http://libexif.sourceforge.net/internals/mnote-olympus-tag_8h-source.html
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Nikon;
 
 use strict;
 use vars qw($VERSION);
-use Image::ExifTool qw(:DataAccess);
+use Image::ExifTool qw(:DataAccess :Utils);
+use Image::ExifTool::Exif;
 
-$VERSION = '1.29';
+$VERSION = '1.35';
 
 %Image::ExifTool::Nikon::Main = (
     PROCESS_PROC => \&Image::ExifTool::Nikon::ProcessNikon,
@@ -43,7 +45,7 @@ $VERSION = '1.29';
     0x0001 => { #2
         # the format differs for different models.  for D70, this is a string '0210',
         # but for the E775 it is binary: "\x00\x01\x00\x00"
-        Name => 'FileSystemVersion',
+        Name => 'FirmwareVersion',
         Writable => 'undef',
         Count => 4,
         # convert to string if binary
@@ -52,7 +54,6 @@ $VERSION = '1.29';
         PrintConv => '$_=$val;s/^(\d{2})/$1\./;s/^0//;$_',
         PrintConvInv => '$_=$val;s/\.//;"0$_"',
     },
-    # 0x0001 - unknown. Always 0210 for D70. Might be a version number? (ref 2)
     0x0002 => {
         # this is the ISO actually used by the camera
         # (may be different than ISO setting if auto)
@@ -148,8 +149,7 @@ $VERSION = '1.29';
         Name => 'SerialNumber',
         Writable => 0,
         Notes => 'Not writable because this value is used as a key to decrypt other information',
-        ValueConv => '$self->{NikonInfo}->{SerialNumber} = $val',
-        ValueConvInv => '$val',
+        RawConv => '$self->{NikonInfo}->{SerialNumber} = $val',
     },
     0x0080 => { Name => 'ImageAdjustment',  Writable => 'string' },
     0x0081 => { Name => 'ToneComp',         Writable => 'string' }, #2
@@ -183,6 +183,7 @@ $VERSION = '1.29';
         Count => 4,
         # short focal, long focal, aperture at short focal, aperture at long focal
         PrintConv => q{
+            $val =~ tr/,/./;    # in case locale is whacky
             my ($a,$b,$c,$d) = split ' ', $val;
             ($a==$b ? $a : "$a-$b") . "mm f/" . ($c==$d ? $c : "$c-$d")
         },
@@ -304,7 +305,7 @@ $VERSION = '1.29';
             Writable => 0,
             SubDirectory => {
                 Start => '$valuePtr + 72',
-                TagTable => 'Image::ExifTool::Nikon::ColorBalance0100',
+                TagTable => 'Image::ExifTool::Nikon::ColorBalance1',
             },
         },
         {
@@ -313,7 +314,7 @@ $VERSION = '1.29';
             Writable => 0,
             SubDirectory => {
                 Start => '$valuePtr + 10',
-                TagTable => 'Image::ExifTool::Nikon::ColorBalance0102',
+                TagTable => 'Image::ExifTool::Nikon::ColorBalance2',
             },
         },
         {
@@ -324,7 +325,31 @@ $VERSION = '1.29';
             # v[0]/v[1] , v[2]/v[3] are the red/blue multipliers.
             SubDirectory => {
                 Start => '$valuePtr + 20',
-                TagTable => 'Image::ExifTool::Nikon::ColorBalance0103',
+                TagTable => 'Image::ExifTool::Nikon::ColorBalance3',
+            },
+        },
+        {
+            Condition => '$$valPt =~ /^0205/', # (D50)
+            Name => 'ColorBalance0205',
+            Writable => 0,
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Nikon::ColorBalance2',
+                ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
+                DecryptStart => 4,
+                DecryptLen => 22, # 324 bytes encrypted, but don't need to decrypt it all
+                DirOffset => 14,
+            },
+        },
+        {
+            Condition => '$$valPt =~ /^02/', # (0204=D2X,0206=D2Hs,0207=D200)
+            Name => 'ColorBalance02',
+            Writable => 0,
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::Nikon::ColorBalance2',
+                ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
+                DecryptStart => 284,
+                DecryptLen => 14, # 324 bytes encrypted, but don't need to decrypt it all
+                DirOffset => 6,
             },
         },
         {
@@ -347,17 +372,19 @@ $VERSION = '1.29';
                 TagTable => 'Image::ExifTool::Nikon::LensData01',
             },
         },
-        # note: his information is encrypted if the version is 02xx
+        # note: this information is encrypted if the version is 02xx
         { #8
             Condition => '$$valPt =~ /^0201/',
             Name => 'LensData0201',
             SubDirectory => {
                 TagTable => 'Image::ExifTool::Nikon::LensData01',
                 ProcessProc => \&Image::ExifTool::Nikon::ProcessNikonEncrypted,
+                DecryptStart => 4,
             },
         },
         {
             Name => 'LensDataUnknown',
+            Writable => 0,
         },
     ],
     # D70 guessing here
@@ -374,12 +401,13 @@ $VERSION = '1.29';
         PrintConvInv => '$val=~tr/a-zA-Z/ /;$val',
     },
     0x00a0 => { Name => 'SerialNumber',     Writable => 'string' }, #2
+    0x00a2 => { Name => 'ImageDataSize' }, # size of compressed image data plus EOI segment (ref 10)
+    # the sum of 0xa5 and 0xa6 is equal to 0xa7 ShutterCount (D2X,D2Hs,D2H,D200, ref 10)
     0x00a7 => { # Number of shots taken by camera so far (ref 2)
         Name => 'ShutterCount',
         Writable => 0,
         Notes => 'Not writable because this value is used as a key to decrypt other information',
-        ValueConv => '$self->{NikonInfo}->{ShutterCount} = $val',
-        ValueConvInv => '$val',
+        RawConv => '$self->{NikonInfo}->{ShutterCount} = $val',
     },
     0x00a9 => { #2
         Name => 'ImageOptimization',
@@ -400,12 +428,20 @@ $VERSION = '1.29';
     # Capture, the data for this tag extends 4 bytes past the end of the maker notes.
     # Very odd.  I hope these 4 bytes aren't useful because they will get lost by any
     # utility that blindly copies the maker notes (not ExifTool) - PH
+    0x0e01 => {
+        Name => 'NikonCaptureData',
+        Writable => 0,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::NikonCapture::Main',
+        },
+    },
+    0x0e09 => 'NikonCaptureVersion', #12
     # 0x0e0e is in D70 Nikon Capture files (not out-of-the-camera D70 files) - PH
     0x0e0e => { #PH
         Name => 'NikonCaptureOffsets',
         Writable => 0,
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Nikon::NikonCaptureOffsets',
+            TagTable => 'Image::ExifTool::Nikon::CaptureOffsets',
             Validate => '$val =~ /^0100/',
             Start => '$valuePtr + 4',
         },
@@ -413,15 +449,16 @@ $VERSION = '1.29';
 );
 
 # ref PH
-%Image::ExifTool::Nikon::NikonCaptureOffsets = (
-    PROCESS_PROC => \&Image::ExifTool::Nikon::ProcessNikonCaptureOffsets,
+%Image::ExifTool::Nikon::CaptureOffsets = (
+    PROCESS_PROC => \&ProcessNikonCaptureOffsets,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     1 => 'IFD0_Offset',
     2 => 'PreviewIFD_Offset',
     3 => 'SubIFD_Offset',
 );
 
-%Image::ExifTool::Nikon::ColorBalance0100 = (
+# ref 4
+%Image::ExifTool::Nikon::ColorBalance1 = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
@@ -445,7 +482,8 @@ $VERSION = '1.29';
     },
 );
 
-%Image::ExifTool::Nikon::ColorBalance0102 = (
+# ref 4
+%Image::ExifTool::Nikon::ColorBalance2 = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
@@ -467,7 +505,8 @@ $VERSION = '1.29';
     },
 );
 
-%Image::ExifTool::Nikon::ColorBalance0103 = (
+# ref 4
+%Image::ExifTool::Nikon::ColorBalance3 = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
@@ -487,7 +526,7 @@ $VERSION = '1.29';
     },
 );
 
-%Image::ExifTool::Nikon::MakerNotesB = (
+%Image::ExifTool::Nikon::Type2 = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
     WRITABLE => 1,
@@ -668,6 +707,7 @@ my %nikonLensIDs = (
     129 => 'AF-S VR Nikkor 200mm f/2G IF-ED',
     130 => 'AF-S VR Nikkor 300mm f/2.8G IF-ED',
     137 => 'AF-S DX Zoom-Nikkor 55-200mm f/4-5.6G ED',
+    139 => 'AF-S VR Zoom-Nikkor 18-200mm f/3.5-5.6G', #10
     140 => 'AF-S DX Zoom-Nikkor 18-55mm f/3.5-5.6G ED',
 );
 
@@ -862,11 +902,9 @@ my @xlat = (
 #         4) optional start offset (default 0)
 #         5) optional number of bytes to decode (default to the end of the data)
 # Returns: Decrypted data block
-sub DecryptNikonData($$$;$$)
+sub Decrypt($$$;$$)
 {
     my ($dataPt, $serial, $count, $start, $len) = @_;
-    # patch for pre-production D50 (ref 8)
-    $serial =~ /^\d+$/ or $serial = 0x22;
     $start or $start = 0;
     my $end = $len ? $start + $len : length($$dataPt);
     my $i;
@@ -887,18 +925,36 @@ sub DecryptNikonData($$$;$$)
 }
 
 #------------------------------------------------------------------------------
-# process Nikon IFD
-# Inputs: 0) ExifTool object reference, 1) pointer to tag table
-#         2) reference to directory information
+# process Nikon Encrypted data block
+# Inputs: 0) ExifTool object reference, 1) reference to directory information
+#         2) pointer to tag table
+# Returns: 1 on success
+sub ProcessNikonEncrypted($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    # get the encrypted directory data
+    my $buff = substr(${$$dirInfo{DataPt}}, $$dirInfo{DirStart}, $$dirInfo{DirLen});
+    # save it until we have enough information to decrypt it later
+    push @{$exifTool->{NikonInfo}->{Encrypted}}, [ $tagTablePtr, $buff, $$dirInfo{TagInfo}];
+    if ($exifTool->Options('Verbose')) {
+        my $indent = substr($exifTool->{INDENT}, 0, -2);
+        print $indent, "[$dirInfo->{TagInfo}->{Name} directory to be decrypted later]\n";
+    }
+    return 1;
+}
+
+#------------------------------------------------------------------------------
+# process Nikon Capture Offsets IFD (ref PH)
+# Inputs: 0) ExifTool object reference, 1) reference to directory information
+#         2) pointer to tag table
 # Returns: 1 on success
 # Notes: This isn't a normal IFD, but is close...
 sub ProcessNikonCaptureOffsets($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
-    my $dataPt = $dirInfo->{DataPt};
-    my $dirStart = $dirInfo->{DirStart};
-    my $dirLen = $dirInfo->{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirStart = $$dirInfo{DirStart};
+    my $dirLen = $$dirInfo{DirLen};
     my $success = 0;
     return 0 unless $dirLen > 2;
     my $count = Get16u($dataPt, $dirStart);
@@ -908,53 +964,30 @@ sub ProcessNikonCaptureOffsets($$$)
         my $pos = $dirStart + 12 * $index + 2;
         my $tagID = Get32u($dataPt, $pos);
         my $value = Get32u($dataPt, $pos + 4);
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tagID);
-        if ($verbose) {
-            $exifTool->VerboseInfo($tagID, $tagInfo,
-                'Table'  => $tagTablePtr,
-                'Index'  => $index,
-                'Value'  => $value,
-                'DataPt' => $dataPt,
-                'Size'   => 12,
-                'Start'  => $pos,
-            );
-        }
-        next unless $tagInfo;
-        $exifTool->FoundTag($tagInfo, $value);
-        $success = 1;
+        $exifTool->HandleTag($tagTablePtr, $tagID, $value,
+            Index  => $index,
+            DataPt => $dataPt,
+            Start  => $pos,
+            Size   => 12,
+        ) and $success = 1;
     }
     return $success;
 }
 
-#------------------------------------------------------------------------------
-# process Nikon Encrypted data block
-# Inputs: 0) ExifTool object reference, 1) pointer to tag table
-#         2) reference to directory information
-# Returns: 1 on success
-sub ProcessNikonEncrypted($$$)
-{
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
-    # get the encrypted directory data
-    my $buff = substr(${$$dirInfo{DataPt}}, $$dirInfo{DirStart}, $$dirInfo{DirLen});
-    # save it until we have enough information to decrypt it later
-    push @{$exifTool->{NikonInfo}->{Encrypted}}, [ $tagTablePtr, $buff ];
-    return 1;
-}
-
 #------------------------------------------------------------------------------
 # Process Nikon Makernotes directory
 # Inputs: 0) ExifTool object reference
-#         1) Pointer to tag table for this directory
-#         2) Reference to directory information hash
+#         1) Reference to directory information hash
+#         2) Pointer to tag table for this directory
 # Returns: 1 on success, otherwise returns 0 and sets a Warning
 sub ProcessNikon($$$)
 {
-    my ($exifTool, $tagTablePtr, $dirInfo) = @_;
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     my $verbose = $exifTool->Options('Verbose');
     my $nikonInfo = $exifTool->{NikonInfo} = { };
     my @encrypted;  # list to save encrypted data
     $$nikonInfo{Encrypted} = \@encrypted;
-    my $rtnVal = Image::ExifTool::Exif::ProcessExif($exifTool, $tagTablePtr, $dirInfo);
+    my $rtnVal = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
     # process any encrypted information we found
     my $encryptedDir;
     if (@encrypted) {
@@ -965,15 +998,48 @@ sub ProcessNikon($$$)
             undef @encrypted;
         }
         foreach $encryptedDir (@encrypted) {
-            my ($subTablePtr, $data) = @$encryptedDir;
-            $data = DecryptNikonData(\$data, $serial, $count, 4);
+            my ($subTablePtr, $data, $tagInfo) = @$encryptedDir;
+            my ($start, $len, $offset);
+            if ($tagInfo and $$tagInfo{SubDirectory}) {
+                $start = $tagInfo->{SubDirectory}->{DecryptStart};
+                $len = $tagInfo->{SubDirectory}->{DecryptLen};
+                $offset = $tagInfo->{SubDirectory}->{DirOffset};
+            }
+            $start or $start = 0;
+            if (defined $offset) {
+                # offset, if specified, is releative to start of encrypted data
+                $offset += $start;
+            } else {
+                $offset = 0;
+            }
+            my $maxLen = length($data) - $start;
+            if ($len) {
+                $len = $maxLen if $len > $maxLen;
+            } else {
+                $len = $maxLen;
+            }
+            # use fixed serial numbers if no good serial number found
+            unless ($serial =~ /^\d+$/) {
+                if ($exifTool->{CameraModel} =~ /\bD200$/) {
+                    $serial = 0x60; # D200 (ref 10)
+                } else {
+                    $serial = 0x22; # D50 (ref 8)
+                }
+            }
+            $data = Decrypt(\$data, $serial, $count, $start, $len);
             my %subdirInfo = (
                 DataPt   => \$data,
-                DirStart => 0,
+                DirStart => $offset,
                 DirLen   => length($data),
             );
+            if ($verbose > 2) {
+                $exifTool->VerboseDir("Decrypted $$tagInfo{Name}");
+                my %parms = ( Prefix => $exifTool->{INDENT} );
+                $parms{MaxLen} = 96 unless $verbose > 3;
+                Image::ExifTool::HexDump(\$data, undef, %parms);
+            }
             # process the decrypted information
-            $exifTool->ProcessBinaryData($subTablePtr, \%subdirInfo);
+            $exifTool->ProcessBinaryData(\%subdirInfo, $subTablePtr);
         }
     }
     delete $exifTool->{NikonInfo};
@@ -986,7 +1052,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::Nikon - Definitions for Nikon EXIF maker notes
+Image::ExifTool::Nikon - Nikon EXIF maker notes tags
 
 =head1 SYNOPSIS
 
@@ -1027,6 +1093,7 @@ out some Nikon tags.
 =head1 SEE ALSO
 
 L<Image::ExifTool::TagNames/Nikon Tags>,
+L<Image::ExifTool::TagNames/NikonCapture Tags>,
 L<Image::ExifTool(3pm)|Image::ExifTool>
 
 =cut
