@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         PDF.pm
 #
-# Description:  Routines for reading PDF files
+# Description:  Read PDF meta information
 #
 # Revisions:    07/11/05 - P. Harvey Created
 #               07/25/05 - P. Harvey Add support for encrypted documents
@@ -693,7 +693,7 @@ sub DecodeStream($$)
             my $inflate = Compress::Zlib::inflateInit();
             my ($buff, $stat);
             $inflate and ($buff, $stat) = $inflate->inflate($$dict{stream});
-            if ($stat == 1) {
+            if ($inflate and $stat == Compress::Zlib::Z_STREAM_END()) {
                 $$dict{stream} = $buff;
                 # move Filter to prevent double decoding
                 $$dict{oldFilter} = $$dict{Filter};
@@ -790,7 +790,8 @@ sub RC4Crypt($$)
 
 #------------------------------------------------------------------------------
 # Initialize decryption
-# Inputs: 0) Encrypt dictionary reference, 1) ID from file trailer dictionary
+# Inputs: 0) ExifTool object reference, 1) Encrypt dictionary reference,
+#         2) ID from file trailer dictionary
 # Returns: error string or undef on success
 sub DecryptInit($$$)
 {
@@ -1063,19 +1064,19 @@ sub ProcessDict($$$$;$)
         Parent => 'PDF',
     );
     my $subTablePtr = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
-    unless ($exifTool->ProcessTagTable($subTablePtr, \%dirInfo)) {
+    unless ($exifTool->ProcessDirectory(\%dirInfo, $subTablePtr)) {
         $exifTool->Warn("Error processing $$tagInfo{Name} information");
     }
 }
 
 #------------------------------------------------------------------------------
 # Extract information from PDF file
-# Inputs: 0) ExifTool object reference
+# Inputs: 0) ExifTool object reference, 1) dirInfo reference
 # Returns: 0 if not a PDF file, 1 on success, otherwise a negative error number
-sub ReadPDF($)
+sub ReadPDF($$)
 {
-    my $exifTool = shift;
-    my $raf = $exifTool->{RAF};
+    my ($exifTool, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
     my $verbose = $exifTool->Options('Verbose');
     my ($data, $encrypt, $id);
 #
@@ -1219,14 +1220,14 @@ my %pdfWarning = (
 
 #------------------------------------------------------------------------------
 # Extract information from PDF file
-# Inputs: 0) ExifTool object reference
+# Inputs: 0) ExifTool object reference, 1) dirInfo reference
 # Returns: 1 if this was a valid PDF file
-sub PdfInfo($)
+sub ProcessPDF($$)
 {
-    my $exifTool = shift;
+    my ($exifTool, $dirInfo) = @_;
 
     my $oldsep = $/;
-    my $result = ReadPDF($exifTool);
+    my $result = ReadPDF($exifTool, $dirInfo);
     $/ = $oldsep;   # restore input record separator in case it was changed
     if ($result < 0) {
         $exifTool->Warn($pdfWarning{$result}) if $pdfWarning{$result};
@@ -1247,7 +1248,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::PDF - Routines for reading PDF files
+Image::ExifTool::PDF - Read PDF meta information
 
 =head1 SYNOPSIS
 
